@@ -2,12 +2,10 @@ from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from math import radians, sin, cos
 from math import pi as π
-from typing import Tuple, Union
+from typing import Tuple
 
-from numba import njit
 import numpy as np
-
-from zodipy._functions import interplanetary_temperature, blackbody_emission
+from numpy.typing import NDArray
 
 
 @dataclass
@@ -28,7 +26,7 @@ class Component(ABC):
         z-offset from the Sun in helliocentric ecliptic cartesian coordinates.
     i
         Inclination with respect to the ecliptic plane [deg].
-    Ω
+    Omega
         Ascending node [deg].
     """
 
@@ -36,17 +34,21 @@ class Component(ABC):
     y_0: float
     z_0: float
     i: float
-    Ω: float
+    Omega: float
 
     def __post_init__(self) -> None:
         self.i = radians(self.i)
-        self.Ω = radians(self.Ω)
-        self.X_component = np.expand_dims([self.x_0, self.y_0, self.z_0], axis=1)
+        self.Omega = radians(self.Omega)
+        self.X_0 = np.expand_dims([self.x_0, self.y_0, self.z_0], axis=1)
 
     @abstractmethod
-    def get_density(
-        self, R_prime: np.ndarray, Z_prime: np.ndarray, *, θ_prime: np.ndarray
-    ) -> np.ndarray:
+    def compute_density(
+        self,
+        R_prime: NDArray[np.float64],
+        Z_prime: NDArray[np.float64],
+        *,
+        θ_prime: NDArray[np.float64],
+    ) -> NDArray[np.float64]:
         """Returns the dust density at a shell around the observer.
 
         Parameters
@@ -68,17 +70,9 @@ class Component(ABC):
             Z_prime, and θ_prime.
         """
 
-    @staticmethod
-    @njit
-    def get_coordinates(
-        R_comp: Union[float, np.ndarray],
-        X_observer: np.ndarray,
-        X_earth: np.ndarray,
-        X_unit: np.ndarray,
-        X_component: np.ndarray,
-        Ω_component: float,
-        i_component: float,
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
+    def get_primed_coordinates(
+        self, X_helio: NDArray[np.float64], X_earth: NDArray[np.float64]
+    ) -> Tuple[NDArray[np.float64], NDArray[np.float64], NDArray[np.float64]]:
         """Returns a set of coordinates for a component.
 
         Given a set of heliocentric ecliptic positions in space given by the
@@ -90,30 +84,14 @@ class Component(ABC):
 
         Parameters
         ----------
-        R_comp
-            Distance R to a shell centered on the observer at which we want
-            to evaluate the Zodiacal emission at.
-        X_observer
-            Heliocentric ecliptic cartesian coordinates of the observer.
+        X_helio
+            Heliocentric ecliptic cartesian coordinates of each considered
+            pixel.
         X_earth
             Heliocentric ecliptic cartesian coordinates of the Earth.
-        X_unit
-            Heliocentric ecliptic cartesian unit vectors pointing to each 
-            position in space we that we consider.
-        X_component
-            Heliocentric ecliptic cartesian off-set of the component
-            (x_0, y_0, z_0).
-        Ω_component
-            Ascending node of the component.
-        i_component
-            Inclination of the component.
 
         Returns
         -------
-        R_helio
-            Array of distances corresponding to discrete points along a
-            line-of-sight for a shell surrounding the observer in heliocentric
-            ecliptic coordinates.
         R_prime
             Array of distances corresponding to discrete points along a
             line-of-sight for a shell surrounding the observer in the primed
@@ -126,80 +104,57 @@ class Component(ABC):
             line-of-sight describe by `R_prime` and the Earth.
         """
 
-        X_helio = R_comp * X_unit + X_observer
-        R_helio = np.sqrt(X_helio[0] ** 2 + X_helio[1] ** 2 + X_helio[2] ** 2)
-
-        X_prime = X_helio - X_component
-        R_prime = np.sqrt(X_prime[0] ** 2 + X_prime[1] ** 2 + X_prime[2] ** 2)
+        X_prime = X_helio - self.X_0
+        R_prime = np.linalg.norm(X_prime, axis=0)
 
         Z_prime = (
-            X_prime[0] * sin(Ω_component) * sin(i_component)
-            - X_prime[1] * cos(Ω_component) * sin(i_component)
-            + X_prime[2] * cos(i_component)
+            X_prime[0] * sin(self.Omega) * sin(self.i)
+            - X_prime[1] * cos(self.Omega) * sin(self.i)
+            + X_prime[2] * cos(self.i)
         )
 
-        X_earth_prime = X_earth - X_component[0]
+        X_earth_prime = np.expand_dims(X_earth, axis=1) - self.X_0
         θ_prime = np.arctan2(X_prime[1], X_prime[0]) - np.arctan2(
             X_earth_prime[1], X_earth_prime[0]
         )
 
-        return R_helio, R_prime, Z_prime, θ_prime
+        return R_prime, Z_prime, θ_prime
 
-    def get_emission(
+    def get_density(
         self,
-        distance_to_shell: Union[float, np.ndarray],
-        observer_position: np.ndarray,
-        earth_position: np.ndarray,
-        unit_vectors: np.ndarray,
-        freq: float,
-    ) -> np.ndarray:
-        """Returns the emission at a shell of distance R from the observer.
-
-        For a description on X_observer, X_earth, X_unit and R, please
-        see the get_coords function.
+        pixel_pos: NDArray[np.float64],
+        earth_pos: NDArray[np.float64],
+        cloud_offset: NDArray[np.float64],
+    ) -> NDArray[np.float64]:
+        """Returns the component density at a shell around the observer.
 
         Parameters
         ----------
-        distance_to_shell
-            Distance R to a shell centered on the observer for which we want
-            to evaluate the Zodiacal emission.
-        observer_position
-            Heliocentric ecliptic cartesian coordinates of the observer.
-        earth_position
+        pixel_pos
+            Heliocentric ecliptic cartesian coordinates of the considered
+            pixels.
+        earth_pos
             Heliocentric ecliptic cartesian coordinates of the Earth.
-        unit_vectors
-            Heliocentric ecliptic cartesian unit vectors pointing to each 
-            position in space we that we consider.
-        freq
-            Frequency at which to evaluate the Zodiacal emission.
 
         Returns
         -------
-        emission
-            Zodiacal emission at
-            Array containing the Zodiacal emission emitted from a shell at
-            distance R from the observer. The shape is (len(R), `NPIX`).
+            Density of a Zodiacal component at a shell around the observer.
         """
 
-        observer_position = np.expand_dims(observer_position, axis=1)
-        R_helio, R_prime, Z_prime, θ_prime = self.get_coordinates(
-            R_comp=distance_to_shell,
-            X_observer=observer_position,
-            X_earth=earth_position,
-            X_unit=unit_vectors,
-            X_component=self.X_component,
-            Ω_component=self.Ω,
-            i_component=self.i,
+        R_prime, Z_prime, θ_prime = self.get_primed_coordinates(
+            X_helio=pixel_pos, X_earth=earth_pos
         )
-        density = self.get_density(
+
+        if isinstance(self, Band):
+            X_cloud = pixel_pos - cloud_offset
+            R_cloud = np.linalg.norm(X_cloud, axis=0)
+            R_prime = R_cloud
+
+        return self.compute_density(
             R_prime=R_prime,
             Z_prime=Z_prime,
             θ_prime=θ_prime,
         )
-        temperature = interplanetary_temperature(R=R_helio)
-        emission = blackbody_emission(T=temperature, ν=freq)
-
-        return emission * density
 
 
 @dataclass
@@ -214,37 +169,39 @@ class Cloud(Component):
     ----------
     n_0
         Density at 1 AU.
-    α
+    alpha
         Radial power-law exponent.
-    β
-        Vertical shape parameter.
-    γ
+    beta
+       Vertical shape parameter.
+    gamma
         Vertical power-law exponent.
-    μ
+    mu
         Widening parameter for the modified fan.
     """
 
     n_0: float
-    α: float
-    β: float
-    γ: float
-    μ: float
+    alpha: float
+    beta: float
+    gamma: float
+    mu: float
 
     def __post_init__(self) -> None:
         super().__post_init__()
 
-    def get_density(self, R_prime: np.ndarray, Z_prime: np.ndarray, **_) -> np.ndarray:
+    def compute_density(
+        self, R_prime: NDArray[np.float64], Z_prime: NDArray[np.float64], **_
+    ) -> NDArray[np.float64]:
         """See base class for documentation."""
 
-        ζ = np.abs(Z_prime) / R_prime
-        μ = self.μ
+        ζ = np.abs(Z_prime / R_prime)
+        μ = self.mu
         g = np.zeros_like(ζ)
 
         condition = ζ < μ
         g[condition] = ζ[condition] ** 2 / (2 * μ)
         g[~condition] = ζ[~condition] - (μ / 2)
 
-        return self.n_0 * R_prime ** -self.α * np.exp(-self.β * g * self.γ)
+        return self.n_0 * R_prime ** -self.alpha * np.exp(-self.beta * g ** self.gamma)
 
 
 @dataclass
@@ -259,36 +216,42 @@ class Band(Component):
     ----------
     n_0
         Density at 3 AU.
-    δ_ζ
+    delta_zeta
         Shape parameter [deg].
     v
         Shape parameter.
     p
         Shape parameter.
-    δ_r
+    delta_r
         Inner radial cutoff.
     """
 
     n_0: float
-    δ_ζ: float
+    delta_zeta: float
     v: float
     p: float
-    δ_r: float
+    delta_r: float
 
     def __post_init__(self) -> None:
         super().__post_init__()
-        self.δ_ζ = radians(self.δ_ζ)
+        self.delta_zeta = radians(self.delta_zeta)
 
-    def get_density(self, R_prime: np.ndarray, Z_prime: np.ndarray, **_) -> np.ndarray:
+    def compute_density(
+        self,
+        R_prime: NDArray[np.float64],
+        Z_prime: NDArray[np.float64],
+        **_,
+    ) -> NDArray[np.float64]:
         """See base class for documentation."""
 
-        ζ = np.abs(Z_prime) / R_prime
-        ζ_over_δ_ζ = ζ / self.δ_ζ
-        term1 = (3 * self.n_0 / R_prime) * np.exp(-((ζ_over_δ_ζ) ** 6))
-        term2 = 1 + ((ζ_over_δ_ζ) ** self.p) / self.v
-        term3 = 1 - np.exp(-((R_prime / self.δ_r) ** 20))
+        ζ = np.abs(Z_prime / R_prime)
+        ζ_over_δ_ζ = ζ / self.delta_zeta
+        term1 = 3 * self.n_0 / R_prime
+        term2 = np.exp(-(ζ_over_δ_ζ ** 6))
+        term3 = self.v + ζ_over_δ_ζ ** self.p
+        term4 = 1 - np.exp(-((R_prime / self.delta_r) ** 20))
 
-        return term1 * term2 * term3
+        return term1 * term2 * term3 * term4
 
 
 @dataclass
@@ -305,25 +268,27 @@ class Ring(Component):
         Density at 1 AU.
     R
         Radius of the peak density.
-    σ_r
+    sigma_r
         Radial dispersion.
-    σ_z
+    sigma_z
         Vertical dispersion.
     """
 
     n_0: float
     R: float
-    σ_r: float
-    σ_z: float
+    sigma_r: float
+    sigma_z: float
 
     def __post_init__(self) -> None:
         super().__post_init__()
 
-    def get_density(self, R_prime: np.ndarray, Z_prime: np.ndarray, **_) -> np.ndarray:
+    def compute_density(
+        self, R_prime: NDArray[np.float64], Z_prime: NDArray[np.float64], **_
+    ) -> NDArray[np.float64]:
         """See base class for documentation."""
 
-        term1 = -(((R_prime - self.R) / self.σ_r) ** 2)
-        term2 = np.abs(Z_prime) / self.σ_z
+        term1 = -((R_prime - self.R) ** 2) / (2 * self.sigma_r ** 2)
+        term2 = np.abs(Z_prime) / self.sigma_z
 
         return self.n_0 * np.exp(term1 - term2)
 
@@ -342,42 +307,46 @@ class Feature(Component):
         Density at 1 AU.
     R
         Radius of the peak density.
-    σ_r
+    sigma_r
         Radial dispersion.
-    σ_z
+    sigma_z
         Vertical dispersion.
     θ
         Longitude with respect to Earth.
-    σ_θ
+    sigma_θ
         Longitude dispersion.
     """
 
     n_0: float
     R: float
-    σ_r: float
-    σ_z: float
-    θ: float
-    σ_θ: float
+    sigma_r: float
+    sigma_z: float
+    theta: float
+    sigma_theta: float
 
     def __post_init__(self) -> None:
         super().__post_init__()
-        self.θ = radians(self.θ)
-        self.σ_θ = radians(self.σ_θ)
+        self.theta = radians(self.theta)
+        self.sigma_theta = radians(self.sigma_theta)
 
-    def get_density(
-        self, R_prime: np.ndarray, Z_prime: np.ndarray, θ_prime: np.ndarray
-    ) -> np.ndarray:
+    def compute_density(
+        self,
+        R_prime: NDArray[np.float64],
+        Z_prime: NDArray[np.float64],
+        θ_prime: NDArray[np.float64],
+        **_,
+    ) -> NDArray[np.float64]:
         """See base class for documentation."""
 
-        Δθ = θ_prime - self.θ
+        Δθ = θ_prime - self.theta
 
         condition1 = Δθ < -π
         condition2 = Δθ > π
         Δθ[condition1] = Δθ[condition1] + 2 * π
         Δθ[condition2] = Δθ[condition2] - 2 * π
 
-        term1 = -(((R_prime - self.R) / self.σ_r) ** 2)
-        term2 = np.abs(Z_prime) / self.σ_z
-        term3 = (Δθ / self.σ_θ) ** 2
+        term1 = -((R_prime - self.R) ** 2) / (2 * self.sigma_r ** 2)
+        term2 = np.abs(Z_prime) / self.sigma_z
+        term3 = Δθ ** 2 / (2 * self.sigma_theta ** 2)
 
         return self.n_0 * np.exp(term1 - term2 - term3)
